package org.unidal.eunit.benchmark.model.entity;

import org.unidal.eunit.benchmark.model.BaseEntity;
import org.unidal.eunit.benchmark.model.IVisitor;

public class MemoryEntity extends BaseEntity<MemoryEntity> {
   private int m_loops;

   private int m_warmups;

   private long m_footprint;

   private long m_permanentFootprint;

   private long m_totalFootprint;

   private long m_gcCount;

   private long m_gcAmount;

   private long m_gcTime;

   public MemoryEntity() {
   }

   @Override
   public void accept(IVisitor visitor) {
      visitor.visitMemory(this);
   }

   @Override
   public boolean equals(Object obj) {
      if (obj instanceof MemoryEntity) {
         MemoryEntity _o = (MemoryEntity) obj;

         if (getLoops() != _o.getLoops()) {
            return false;
         }

         if (getWarmups() != _o.getWarmups()) {
            return false;
         }

         if (getFootprint() != _o.getFootprint()) {
            return false;
         }

         if (getPermanentFootprint() != _o.getPermanentFootprint()) {
            return false;
         }

         if (getTotalFootprint() != _o.getTotalFootprint()) {
            return false;
         }

         if (getGcCount() != _o.getGcCount()) {
            return false;
         }

         if (getGcAmount() != _o.getGcAmount()) {
            return false;
         }

         if (getGcTime() != _o.getGcTime()) {
            return false;
         }


         return true;
      }

      return false;
   }

   public long getFootprint() {
      return m_footprint;
   }

   public long getGcAmount() {
      return m_gcAmount;
   }

   public long getGcCount() {
      return m_gcCount;
   }

   public long getGcTime() {
      return m_gcTime;
   }

   public int getLoops() {
      return m_loops;
   }

   public long getPermanentFootprint() {
      return m_permanentFootprint;
   }

   public long getTotalFootprint() {
      return m_totalFootprint;
   }

   public int getWarmups() {
      return m_warmups;
   }

   @Override
   public int hashCode() {
      int hash = 0;

      hash = hash * 31 + m_loops;
      hash = hash * 31 + m_warmups;
      hash = hash * 31 + (int) (m_footprint ^ (m_footprint >>> 32));
      hash = hash * 31 + (int) (m_permanentFootprint ^ (m_permanentFootprint >>> 32));
      hash = hash * 31 + (int) (m_totalFootprint ^ (m_totalFootprint >>> 32));
      hash = hash * 31 + (int) (m_gcCount ^ (m_gcCount >>> 32));
      hash = hash * 31 + (int) (m_gcAmount ^ (m_gcAmount >>> 32));
      hash = hash * 31 + (int) (m_gcTime ^ (m_gcTime >>> 32));

      return hash;
   }

   @Override
   public void mergeAttributes(MemoryEntity other) {
      m_loops = other.getLoops();

      m_warmups = other.getWarmups();

      m_footprint = other.getFootprint();

      m_permanentFootprint = other.getPermanentFootprint();

      m_totalFootprint = other.getTotalFootprint();

      m_gcCount = other.getGcCount();

      m_gcAmount = other.getGcAmount();

      m_gcTime = other.getGcTime();
   }

   public MemoryEntity setFootprint(long footprint) {
      m_footprint = footprint;
      return this;
   }

   public MemoryEntity setGcAmount(long gcAmount) {
      m_gcAmount = gcAmount;
      return this;
   }

   public MemoryEntity setGcCount(long gcCount) {
      m_gcCount = gcCount;
      return this;
   }

   public MemoryEntity setGcTime(long gcTime) {
      m_gcTime = gcTime;
      return this;
   }

   public MemoryEntity setLoops(int loops) {
      m_loops = loops;
      return this;
   }

   public MemoryEntity setPermanentFootprint(long permanentFootprint) {
      m_permanentFootprint = permanentFootprint;
      return this;
   }

   public MemoryEntity setTotalFootprint(long totalFootprint) {
      m_totalFootprint = totalFootprint;
      return this;
   }

   public MemoryEntity setWarmups(int warmups) {
      m_warmups = warmups;
      return this;
   }

}
