package org.unidal.eunit.benchmark.model.entity;

import static org.unidal.eunit.benchmark.model.Constants.ATTR_TYPE;
import static org.unidal.eunit.benchmark.model.Constants.ENTITY_SUITE;

import java.util.ArrayList;
import java.util.List;

import org.unidal.eunit.benchmark.model.BaseEntity;
import org.unidal.eunit.benchmark.model.IVisitor;

public class SuiteEntity extends BaseEntity<SuiteEntity> {
   private Class<?> m_type;

   private List<CaseEntity> m_cases = new ArrayList<CaseEntity>();

   private CpuEntity m_cpu;

   private MemoryEntity m_memory;

   public SuiteEntity() {
   }

   public SuiteEntity(Class<?> type) {
      m_type = type;
   }

   @Override
   public void accept(IVisitor visitor) {
      visitor.visitSuite(this);
   }

   public SuiteEntity addCase(CaseEntity _case) {
      m_cases.add(_case);
      return this;
   }

   @Override
   public boolean equals(Object obj) {
      if (obj instanceof SuiteEntity) {
         SuiteEntity _o = (SuiteEntity) obj;

         if (!equals(getType(), _o.getType())) {
            return false;
         }

         return true;
      }

      return false;
   }

   public CaseEntity findCase(String name) {
      for (CaseEntity _case : m_cases) {
         if (!equals(_case.getName(), name)) {
            continue;
         }

         return _case;
      }

      return null;
   }

   public List<CaseEntity> getCases() {
      return m_cases;
   }

   public CpuEntity getCpu() {
      return m_cpu;
   }

   public MemoryEntity getMemory() {
      return m_memory;
   }

   public Class<?> getType() {
      return m_type;
   }

   @Override
   public int hashCode() {
      int hash = 0;

      hash = hash * 31 + (m_type == null ? 0 : m_type.hashCode());

      return hash;
   }

   @Override
   public void mergeAttributes(SuiteEntity other) {
      assertAttributeEquals(other, ENTITY_SUITE, ATTR_TYPE, m_type, other.getType());

   }

   public CaseEntity removeCase(String name) {
      int len = m_cases.size();

      for (int i = 0; i < len; i++) {
         CaseEntity _case = m_cases.get(i);

         if (!equals(_case.getName(), name)) {
            continue;
         }

         return m_cases.remove(i);
      }

      return null;
   }

   public SuiteEntity setCpu(CpuEntity cpu) {
      m_cpu = cpu;
      return this;
   }

   public SuiteEntity setMemory(MemoryEntity memory) {
      m_memory = memory;
      return this;
   }

   public SuiteEntity setType(Class<?> type) {
      m_type = type;
      return this;
   }

}
