package org.unidal.eunit.benchmark.model.transform;

import static org.unidal.eunit.benchmark.model.Constants.ATTR_CPU_FIRST_TIME;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_CPU_TIME;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_CPU_TOTAL_TIME;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_ELAPSED_FIRST_TIME;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_ELAPSED_TIME;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_ELAPSED_TOTAL_TIME;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_FOOTPRINT;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_GC_AMOUNT;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_GC_COUNT;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_GC_TIME;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_LOOPS;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_NAME;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_PERMANENT_FOOTPRINT;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_TOTAL_FOOTPRINT;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_TYPE;
import static org.unidal.eunit.benchmark.model.Constants.ATTR_WARMUPS;
import static org.unidal.eunit.benchmark.model.Constants.ELEMENT_END_AT;
import static org.unidal.eunit.benchmark.model.Constants.ELEMENT_START_AT;
import static org.unidal.eunit.benchmark.model.Constants.ENTITY_BENCHMARK;
import static org.unidal.eunit.benchmark.model.Constants.ENTITY_CASE;
import static org.unidal.eunit.benchmark.model.Constants.ENTITY_CPU;
import static org.unidal.eunit.benchmark.model.Constants.ENTITY_MEMORY;
import static org.unidal.eunit.benchmark.model.Constants.ENTITY_SUITE;

import java.lang.reflect.Array;
import java.util.Collection;

import org.unidal.eunit.benchmark.model.IEntity;
import org.unidal.eunit.benchmark.model.IVisitor;
import org.unidal.eunit.benchmark.model.entity.BenchmarkEntity;
import org.unidal.eunit.benchmark.model.entity.CaseEntity;
import org.unidal.eunit.benchmark.model.entity.CpuEntity;
import org.unidal.eunit.benchmark.model.entity.MemoryEntity;
import org.unidal.eunit.benchmark.model.entity.SuiteEntity;

public class DefaultXmlBuilder implements IVisitor {

   private IVisitor m_visitor = this;

   private int m_level;

   private StringBuilder m_sb;

   private boolean m_compact;

   public DefaultXmlBuilder() {
      this(false);
   }

   public DefaultXmlBuilder(boolean compact) {
      this(compact, new StringBuilder(4096));
   }

   public DefaultXmlBuilder(boolean compact, StringBuilder sb) {
      m_compact = compact;
      m_sb = sb;
      m_sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n");
   }

   public String buildXml(IEntity<?> entity) {
      entity.accept(m_visitor);
      return m_sb.toString();
   }

   protected void endTag(String name) {
      m_level--;

      indent();
      m_sb.append("</").append(name).append(">\r\n");
   }

   protected String escape(Object value) {
      return escape(value, false);
   }
   
   protected String escape(Object value, boolean text) {
      if (value == null) {
         return null;
      }

      String str = toString(value);
      int len = str.length();
      StringBuilder sb = new StringBuilder(len + 16);

      for (int i = 0; i < len; i++) {
         final char ch = str.charAt(i);

         switch (ch) {
         case '<':
            sb.append("&lt;");
            break;
         case '>':
            sb.append("&gt;");
            break;
         case '&':
            sb.append("&amp;");
            break;
         case '"':
            if (!text) {
               sb.append("&quot;");
               break;
            }
         default:
            sb.append(ch);
            break;
         }
      }

      return sb.toString();
   }
   
   protected void indent() {
      if (!m_compact) {
         for (int i = m_level - 1; i >= 0; i--) {
            m_sb.append("   ");
         }
      }
   }

   protected void startTag(String name) {
      startTag(name, false, null);
   }
   
   protected void startTag(String name, boolean closed, java.util.Map<String, String> dynamicAttributes, Object... nameValues) {
      startTag(name, null, closed, dynamicAttributes, nameValues);
   }

   protected void startTag(String name, java.util.Map<String, String> dynamicAttributes, Object... nameValues) {
      startTag(name, null, false, dynamicAttributes, nameValues);
   }

   protected void startTag(String name, Object text, boolean closed, java.util.Map<String, String> dynamicAttributes, Object... nameValues) {
      indent();

      m_sb.append('<').append(name);

      int len = nameValues.length;

      for (int i = 0; i + 1 < len; i += 2) {
         Object attrName = nameValues[i];
         Object attrValue = nameValues[i + 1];

         if (attrValue != null) {
            m_sb.append(' ').append(attrName).append("=\"").append(escape(attrValue)).append('"');
         }
      }

      if (dynamicAttributes != null) {
         for (java.util.Map.Entry<String, String> e : dynamicAttributes.entrySet()) {
            m_sb.append(' ').append(e.getKey()).append("=\"").append(escape(e.getValue())).append('"');
         }
      }

      if (text != null && closed) {
         m_sb.append('>');
         m_sb.append(escape(text, true));
         m_sb.append("</").append(name).append(">\r\n");
      } else {
         if (closed) {
            m_sb.append('/');
         } else {
            m_level++;
         }
   
         m_sb.append(">\r\n");
      }
   }

   @SuppressWarnings("unchecked")
   protected String toString(Object value) {
      if (value instanceof String) {
         return (String) value;
      } else if (value instanceof Collection) {
         Collection<Object> list = (Collection<Object>) value;
         StringBuilder sb = new StringBuilder(32);
         boolean first = true;

         for (Object item : list) {
            if (first) {
               first = false;
            } else {
               sb.append(',');
            }

            if (item != null) {
               sb.append(item);
            }
         }

         return sb.toString();
      } else if (value.getClass().isArray()) {
         int len = Array.getLength(value);
         StringBuilder sb = new StringBuilder(32);
         boolean first = true;

         for (int i = 0; i < len; i++) {
            Object item = Array.get(value, i);

            if (first) {
               first = false;
            } else {
               sb.append(',');
            }

            if (item != null) {
               sb.append(item);
            }
         }
		
         return sb.toString();
      }
 
      return String.valueOf(value);
   }

   protected void tagWithText(String name, String text, Object... nameValues) {
      if (text == null) {
         return;
      }
      
      indent();

      m_sb.append('<').append(name);

      int len = nameValues.length;

      for (int i = 0; i + 1 < len; i += 2) {
         Object attrName = nameValues[i];
         Object attrValue = nameValues[i + 1];

         if (attrValue != null) {
            m_sb.append(' ').append(attrName).append("=\"").append(escape(attrValue)).append('"');
         }
      }

      m_sb.append(">");
      m_sb.append(escape(text, true));
      m_sb.append("</").append(name).append(">\r\n");
   }

   protected void element(String name, String text, String defaultValue, boolean escape) {
      if (text == null || text.equals(defaultValue)) {
         return;
      }
      
      indent();
      
      m_sb.append('<').append(name).append(">");
      
      if (escape) {
         m_sb.append(escape(text, true));
      } else {
         m_sb.append("<![CDATA[").append(text).append("]]>");
      }
      
      m_sb.append("</").append(name).append(">\r\n");
   }

   @Override
   public void visitBenchmark(BenchmarkEntity benchmark) {
      startTag(ENTITY_BENCHMARK, null);

      if (!benchmark.getSuites().isEmpty()) {
         for (SuiteEntity suite : benchmark.getSuites()) {
            suite.accept(m_visitor);
         }
      }

      endTag(ENTITY_BENCHMARK);
   }

   @Override
   public void visitCase(CaseEntity _case) {
      startTag(ENTITY_CASE, null, ATTR_NAME, _case.getName());

      element(ELEMENT_START_AT, _case.getStartAt(), null,  true);

      element(ELEMENT_END_AT, _case.getEndAt(), null,  true);

      if (_case.getCpu() != null) {
         _case.getCpu().accept(m_visitor);
      }

      if (_case.getMemory() != null) {
         _case.getMemory().accept(m_visitor);
      }

      endTag(ENTITY_CASE);
   }

   @Override
   public void visitCpu(CpuEntity cpu) {
      startTag(ENTITY_CPU, true, null, ATTR_LOOPS, cpu.getLoops(), ATTR_WARMUPS, cpu.getWarmups(), ATTR_CPU_TIME, cpu.getCpuTime(), ATTR_CPU_FIRST_TIME, cpu.getCpuFirstTime(), ATTR_CPU_TOTAL_TIME, cpu.getCpuTotalTime(), ATTR_ELAPSED_TIME, cpu.getElapsedTime(), ATTR_ELAPSED_FIRST_TIME, cpu.getElapsedFirstTime(), ATTR_ELAPSED_TOTAL_TIME, cpu.getElapsedTotalTime());
   }

   @Override
   public void visitMemory(MemoryEntity memory) {
      startTag(ENTITY_MEMORY, true, null, ATTR_LOOPS, memory.getLoops(), ATTR_WARMUPS, memory.getWarmups(), ATTR_FOOTPRINT, memory.getFootprint(), ATTR_PERMANENT_FOOTPRINT, memory.getPermanentFootprint(), ATTR_TOTAL_FOOTPRINT, memory.getTotalFootprint(), ATTR_GC_COUNT, memory.getGcCount(), ATTR_GC_AMOUNT, memory.getGcAmount(), ATTR_GC_TIME, memory.getGcTime());
   }

   @Override
   public void visitSuite(SuiteEntity suite) {
      startTag(ENTITY_SUITE, null, ATTR_TYPE, suite.getType() == null ? null : suite.getType().getName());

      if (!suite.getCases().isEmpty()) {
         for (CaseEntity case_ : suite.getCases()) {
            case_.accept(m_visitor);
         }
      }

      if (suite.getCpu() != null) {
         suite.getCpu().accept(m_visitor);
      }

      if (suite.getMemory() != null) {
         suite.getMemory().accept(m_visitor);
      }

      endTag(ENTITY_SUITE);
   }
}
