package org.unidal.web.mvc.model.entity;

import java.util.ArrayList;
import java.util.List;

import org.unidal.web.mvc.model.BaseEntity;
import org.unidal.web.mvc.model.IVisitor;

public class PayloadModel extends BaseEntity<PayloadModel> {
   private Class<?> m_payloadClass;

   private List<PayloadFieldModel> m_fields = new ArrayList<PayloadFieldModel>();

   private List<PayloadPathModel> m_paths = new ArrayList<PayloadPathModel>();

   private List<PayloadObjectModel> m_objects = new ArrayList<PayloadObjectModel>();

   public PayloadModel() {
   }

   @Override
   public void accept(IVisitor visitor) {
      visitor.visitPayload(this);
   }

   public PayloadModel addField(PayloadFieldModel field) {
      m_fields.add(field);
      return this;
   }

   public PayloadModel addObject(PayloadObjectModel object) {
      m_objects.add(object);
      return this;
   }

   public PayloadModel addPath(PayloadPathModel path) {
      m_paths.add(path);
      return this;
   }

   @Override
   public boolean equals(Object obj) {
      if (obj instanceof PayloadModel) {
         PayloadModel _o = (PayloadModel) obj;
         Class<?> payloadClass = _o.getPayloadClass();
         List<PayloadFieldModel> fields = _o.getFields();
         List<PayloadPathModel> paths = _o.getPaths();
         List<PayloadObjectModel> objects = _o.getObjects();
         boolean result = true;

         result &= (m_payloadClass == payloadClass || m_payloadClass != null && m_payloadClass.equals(payloadClass));
         result &= (m_fields == fields || m_fields != null && m_fields.equals(fields));
         result &= (m_paths == paths || m_paths != null && m_paths.equals(paths));
         result &= (m_objects == objects || m_objects != null && m_objects.equals(objects));

         return result;
      }

      return false;
   }

   public List<PayloadFieldModel> getFields() {
      return m_fields;
   }

   public List<PayloadObjectModel> getObjects() {
      return m_objects;
   }

   public List<PayloadPathModel> getPaths() {
      return m_paths;
   }

   public Class<?> getPayloadClass() {
      return m_payloadClass;
   }

   @Override
   public int hashCode() {
      int hash = 0;

      hash = hash * 31 + (m_payloadClass == null ? 0 : m_payloadClass.hashCode());
      hash = hash * 31 + (m_fields == null ? 0 : m_fields.hashCode());
      hash = hash * 31 + (m_paths == null ? 0 : m_paths.hashCode());
      hash = hash * 31 + (m_objects == null ? 0 : m_objects.hashCode());

      return hash;
   }

   @Override
   public void mergeAttributes(PayloadModel other) {
      if (other.getPayloadClass() != null) {
         m_payloadClass = other.getPayloadClass();
      }
   }

   public PayloadModel setPayloadClass(Class<?> payloadClass) {
      m_payloadClass = payloadClass;
      return this;
   }

}
